#!/bin/bash

source install_library.bash
source sources.bash

echo Using ARCHIVE=$ARCHIVE
cd $ARCHIVE

# Basic stuff
install_packages build-essential kernel-package

# Current running kernel...
KBASE=`uname -r`
KROOT=${KBASE%%-*}

# Guess source name based on current kernel
if [ "$LINUX_SOURCE_NAME" == "" ]; then
    LINUX_SOURCE_NAME=linux-source-${KROOT}
fi

# Name of the kernel source archive
TARC=/usr/src/${LINUX_SOURCE_NAME}.tar.bz2

# Are we using the Ubuntu packaged kernel source?
if test $LINUX_SOURCE_URL ; then
    # Download kernel source from somewhere
    archive=`http_dest $LINUX_SOURCE_URL`
    archive_basename=`basename $archive`
    if [ -e "$TARC" ]; then
	echo "Destination location $TARC already exists, skipping download."
    else
	if [ -e "$archive_basename" ]; then
	    echo "Kernel archive found in current dir, moving it..."
	else
	    echo "Downloading kernel: $LINUX_SOURCE_URL."
	    get_http $LINUX_SOURCE_URL > /dev/null
	    [ -e "$archive_basename" ] || exit 1
	fi
        # Move to canonical location
	echo "Moving to $TARC"
	$SUDO mv "$archive" "$TARC"
    fi
else
    # Install kernel source package
    [ -e "$TARC" ] || install_packages $LINUX_SOURCE_PKG
fi

if ! [ -e "$TARC" ]; then
    echo "Make LINUX_SOURCE_NAME match the archive in /usr/src."
    exit 1
fi

DEST=$ARCHIVE/${LINUX_SOURCE_NAME}

if [ -e $DEST ]; then
    echo $DEST exists, skipping extraction.
else
    echo "Untarring $TARC to $DEST (I think)"
    tar -xjf $TARC 
fi

# Go into kernel source tree.
cd $DEST

[ "${BIGPHYSPATCH_FILE:0:1}" == "/" ] || \
    BIGPHYSPATCH_FILE="${ARCHIVE}/${BIGPHYSPATCH_FILE}"

# Apply patch
PATCHOPTS="--forward -p1 -i $BIGPHYSPATCH_FILE"
if patch $PATCHOPTS --dry-run; then
    patch $PATCHOPTS
else
    echo "Patch will not go gently, go manual."
    exit 1
fi

case "$DEFAULT_CONFIG" in
    copy)
	# Copy current system kernel configuration
	CONFIG=/boot/config-$KBASE
	cp $CONFIG .config
	;;
    new)
	# Run menuconfig; user should just 
	make menuconfig
	;;
    *)
	# Ok, we'll use that.
	cp "$DEFAULT_CONFIG" .config
	;;
esac

# Append the bigphys switch.
echo "CONFIG_BIGPHYS_AREA=y" >> .config

# Build
fakeroot make-kpkg clean && \
fakeroot make-kpkg --initrd kernel-image kernel-headers \
    --append-to-version=-bigphys

if [ "$?" == "0" ]; then
    echo 
    echo "Build appears to have worked.  There should be image and header"
    echo "packages in this folder."
    echo
else
    echo
    echo "Build appears to have failed, but check for packages."
    echo
fi

# Go up one level, where .deb should be
cd ..
